<?php

namespace InSegment\WorkersFramework\Drivers\RabbitMq;

use Shavshukov\EventAppBase\Interfaces\ModelInterface;
use Shavshukov\EventAppBase\Model\ArrayModel;
use Bunny\Channel;
use Bunny\Message;

use InSegment\WorkersFramework\Interfaces\QueueMessageInterface;

class QueueMessage implements QueueMessageInterface
{
    /**
     * Channel which has received message and will be used for acknowledgement
     *
     * @var \Bunny\Channel
     */
    protected $channel;
    
    /**
     * Bunny message
     *
     * @var \Bunny\Message
     */
    protected $message;
    
    /**
     * Name of the queue this message came from
     *
     * @var string
     */
    protected $queueName;
    
    /**
     * Constructor
     * 
     * @param \Bunny\Message $message
     * @param \Bunny\Channel $channel
     */
    public function __construct(Message $message, Channel $channel, string $queueName)
    {
        $this->message = $message;
        $this->channel = $channel;
        $this->queueName = $queueName;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getConsumerTag(): ?string
    {
        return $this->message->consumerTag;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getContents(): string
    {
        return $this->message->content;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getHeaders(): ModelInterface
    {
        return ArrayModel::from($this->message->headers);
    }
    
    /**
     * {@inheritDoc}
     */
    public function getQueueName(): string
    {
        return $this->queueName;
    }
    
    /**
     * {@inheritDoc}
     */
    public function acknowlege()
    {
        return $this->channel->ack($this->message);
    }
    
}
