<?php

namespace InSegment\WorkersFramework\Http;

use Shavshukov\EventAppBase\Http\Routing\Router as BaseRouter;
use Shavshukov\EventAppBase\Http\Controller;
use Psr\Http\Message\ServerRequestInterface;
use React\Promise;

use InSegment\WorkersFramework\Traits\AuthenticationCheck;

class Router extends BaseRouter
{
    use AuthenticationCheck;
    
    /**
     * {@inheritDoc}
     */
    public function getRoutes(): array
    {
        return [
            '/' => '/index.html',
            '/401' => ['GET' => ['ErrorController', 'unauthorised']],
            '/403' => ['GET' => ['ErrorController', 'accessDenied']],
            '/404' => ['GET' => ['ErrorController', 'notFound']],
            '/stats' => ['GET' => ['StatsController', 'getStats']],
            '/announce' => ['POST' => ['AnnounceController', 'postAnnounce']]
        ];
    }

    /**
     * {@inheritDoc}
     */
    public function resolveController(string $controllerName): Controller
    {
        $class = "InSegment\\WorkersFramework\\Http\\Controllers\\{$controllerName}";
        return new $class($this->server, $this->app);
    }

    /**
     * {@inheritDoc}
     */
    public function authorise(ServerRequestInterface $request, string $method, string $path): Promise\PromiseInterface
    {
        if ($method === 'GET' && ($path === '/403' || $path === '/404')) {
            return Promise\resolve();
        }
        
        $authorization = $request->getHeader('Authorization');
        $authorizationValid = isset($authorization[0]) && $this->checkAuthString($authorization[0]);
        
        if ($authorizationValid) {
            if ($method === 'GET' && $path === '/401') {
                return Promise\reject(['GET', '/']);
            } else {
                return Promise\resolve();
            }
        } else if ($method === 'GET' && $path === '/401') {
            return Promise\resolve();
        }
        
        return Promise\reject(['GET', '/401']);
    }


}