<?php

namespace InSegment\WorkersFramework\Interfaces;

use InSegment\WorkersFramework\Interfaces\QueueMessageInterface;
use InSegment\WorkersFramework\Models\QueueConfig;

interface QueueDriverInterface
{
    
    /**
     * Open queue for work and get message count
     * 
     * @param \InSegment\WorkersFramework\Models\QueueConfig $configuration
     * @return mixed
     */
    public function openQueueForWork(QueueConfig $configuration);
    
    /**
     * Get message count from queue
     * 
     * @param \InSegment\WorkersFramework\Models\QueueConfig $configuration
     * @return int|\React\Promise\PromiseInterface<int>
     */
    public function getMessageCount(QueueConfig $configuration);
    
    /**
     * Declare queue structure described by configuration
     * 
     * @param \InSegment\WorkersFramework\Models\QueueConfig $configuration
     * @raturn mixed
     */
    public function declareQueueStructure(QueueConfig $configuration);
    
    /**
     * Publish message to the queue
     * 
     * @param \InSegment\WorkersFramework\Interfaces\QueueMessageInterface $message
     * @return mixed
     */
    public function publish(QueueMessageInterface $message);
    
    /**
     * Subscribe to queue and return consumer tag
     * 
     * @param string $queueName
     * @param \InSegment\WorkersFramework\Interfaces\ConsumerInterface $reciever
     * @return mixed
     */
    public function subscribe(string $queueName, ConsumerInterface $reciever);
    
    /**
     * Un-subscribe from channel
     * 
     * @param string $consumerTag
     * @return mixed
     */
    public function unsubscribe(string $consumerTag);
    
}
