<?php

namespace InSegment\WorkersFramework\Queues;

class LoadMeasure
{
    /**
     * Current load
     *
     * @var array
     */
    protected $load = [];
    
    /**
     * Constructor
     * 
     * @param array $names
     */
    public function __construct(array $names)
    {
        foreach ($names as $name) {
            $this->load[$name] = 0;
        }
    }
    
    /**
     * Increase a load of name, if it becomes the new minimum, decrease the load of every name by their common minimum
     * 
     * @param string $incrementName
     * @param int $amount
     * @return null
     */
    public function increment(string $incrementName, int $amount)
    {
        $this->load[$incrementName] += $amount;
        
        if ($this->load[$incrementName] === min($this->load)) {
            foreach (array_keys($this->load) as $name) {
                $this->load[$name] -= $this->load[$incrementName];
            }
        }
    }
    
    /**
     * Remove name from measure
     * 
     * @param string $name
     * @return null
     */
    public function remove(string $name)
    {
        unset($this->load[$name]);
        
        $newMinumum = min($this->load);
        foreach (array_keys($this->load) as $name) {
            $this->load[$name] -= $newMinumum;
        }
    }
    
    /**
     * Add new name for measure
     * 
     * @param string $name
     * @return null
     */
    public function add(string $name)
    {
        if (!isset($this->load[$name])) {
            $this->load[$name] = 0;
        }
    }

    /**
     * Get current load for name
     * 
     * @param string $name
     * @return int
     */
    public function getLoad(string $name): int
    {
        return $this->load[$name];
    }
}
