<?php

namespace InSegment\WorkersFramework\Queues;

use Shavshukov\EventAppBase\Model\ArrayModel;

use InSegment\WorkersFramework\Interfaces\QueueDriverInterface;

class ServiceMessenger
{
    /**
     * Instance of queue driver connected to the channel
     *
     * @var \InSegment\WorkersFramework\Interfaces\QueueDriverInterface
     */
    protected $queueDriver;
    
    /**
     * Name of the service queue
     *
     * @var string
     */
    protected $queueName;
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Interfaces\QueueDriverInterface $queueDriver
     * @param string $queueName
     */
    public function __construct(QueueDriverInterface $queueDriver, string $queueName)
    {
        $this->queueDriver = $queueDriver;
        $this->queueName = $queueName;
    }
    
    /**
     * Send updates to the service queue
     * 
     * @param array $updates
     * @return \React\Promise\PromiseInterface
     */
    public function sendUpdates(array $updates)
    {
        $updates['timestamp'] = microtime(true);
        return $this->queueDriver->publish(new UnpublishedMessage(
            $this->queueName,
            json_encode($updates),
            new ArrayModel()
        ));
    }
    
}
