<?php

namespace InSegment\WorkersFramework\Queues;

use Shavshukov\EventAppBase\Interfaces\ModelInterface;

use InSegment\WorkersFramework\Interfaces\QueueMessageInterface;

class UnpublishedMessage implements QueueMessageInterface
{
    /**
     * Name of the queue to publish message to
     *
     * @var string
     */
    protected $queueName;
    
    /**
     * Payload of the message
     *
     * @var string
     */
    protected $contents;
    
    /**
     * Headers to publish message with
     *
     * @var \Shavshukov\EventAppBase\Interfaces\ModelInterface
     */
    protected $headers;
    
    /**
     * Constructor
     * 
     * @param string $queueName
     * @param string $contents
     * @param \Shavshukov\EventAppBase\Interfaces\ModelInterface $headers
     */
    public function __construct(string $queueName, string $contents, ModelInterface $headers)
    {
        $this->queueName = $queueName;
        $this->contents = $contents;
        $this->headers = $headers;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getConsumerTag(): ?string
    {
        return null;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getContents(): string
    {
        return $this->contents;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getHeaders(): ModelInterface
    {
        return $this->headers;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getQueueName(): string
    {
        return $this->queueName;
    }
    
    /**
     * {@inheritDoc}
     */
    public function acknowlege()
    {
        throw new \Exception('The message was not published - it cannot be acknowledged');
    }
}
