<?php

namespace InSegment\WorkersFramework\Services;

use Shavshukov\EventAppBase\ApplicationInterface;
use React\Promise;

use InSegment\WorkersFramework\Interfaces\QueueDriverInterface;
use InSegment\WorkersFramework\Models\QueueConfig;
use InSegment\WorkersFramework\Queues\Worker;

class Declarator
{
    /**
     * Application instance
     *
     * @var \Shavshukov\EventAppBase\ApplicationInterface
     */
    protected $app;
    
    /**
     * Instance of queue driver connected to the channel
     *
     * @var \InSegment\WorkersFramework\Interfaces\QueueDriverInterface
     */
    protected $queueDriver;
    
    /**
     * Constructor
     * 
     * @param \Shavshukov\EventAppBase\ApplicationInterface $app
     * @param \InSegment\WorkersFramework\Interfaces\QueueDriverInterface $queueDriver
     * @throws \Exception
     */
    public function __construct(ApplicationInterface $app, QueueDriverInterface $queueDriver)
    {
        $this->app = $app;
        $this->queueDriver = $queueDriver;
    }
    
    /**
     * Declare necessary queues
     * 
     * @return \React\Promise\PromiseInterface
     */
    public function declare(): Promise\PromiseInterface
    {
        return $this->declareIncomingQueue()->then(function () {
            $promises = [];
            foreach ($this->app->config('workers', []) as $workerName => $workerConfig) {
                $promises[] = $this->declareQueueForWorker(new Worker($workerName, $workerConfig));
            }
            
            return Promise\all($promises);
        })->then(function () {
            echo "Done.";
        });
    }
    
    /**
     * Declare main input queue
     * 
     * @return \React\Promise\PromiseInterface
     */
    protected function declareIncomingQueue(): Promise\PromiseInterface
    {
        echo "Declaring main incoming and service queues...\n";
        
        $config = new QueueConfig([
            'queueName' => $this->app->config('main_queue'),
            'serviceQueueName' => $this->app->config('service_queue'),
        ]);
        
        $config->check();
        return $this->queueDriver->declareQueueStructure($config);
    }
    
    /**
     * Declare queue for worker
     * 
     * @param \InSegment\WorkersFramework\Queues\Worker $worker
     * @return \React\Promise\PromiseInterface
     */
    protected function declareQueueForWorker(Worker $worker): Promise\PromiseInterface
    {
        echo "Declaring queue: {$worker->getQueueName()} for worker {$worker->getName()}...\n";
        
        return $this->queueDriver->declareQueue($worker->getQueueConfig());
    }
}
