<?php

namespace InSegment\WorkersFramework\Traits;

trait AuthenticationCheck
{
    /**
     * Valid authentication string already prepared to check against
     *
     * @var string
     */
    private $validAuthString;
    
    /**
     * Check authentication string against valid authentication string
     * 
     * @param string $authString
     * @return bool
     */
    public function checkAuthString(string $authString): bool
    {
        return $authString === $this->getValidAuthString();
    }
    
    /**
     * Get prepared valid string to check authentication against
     * 
     * @return string
     */
    private function getValidAuthString(): string
    {
        if (isset($this->validAuthString)) {
            return $this->validAuthString;
        }
        
        $user = $this->app->config('http_user', '');
        $password = $this->app->config('http_password', '');
        return $this->validAuthString = 'Basic ' . base64_encode("{$user}:{$password}");
    }
    
}
